#! /usr/bin/env python3

'''
 caesar3.py
 Verschluesselung durch Substitution
 nach Caesar
 Buchstaben, Satzzeichen und Zaehlen werden ersetzt durch Nachbarzeichen (das x-te rechts)
anders wie bei den Vorgaengerprogrammen (caesar1.py und caesar2.py) will
ich hier das 'neue'/'andere' string-Modul nutzen.
>>> print (string.ascii_letters)
abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ
>>>

okay - hier ist die Reihenfolge umgedreht ('A' ist doch vor 'a' in der Tabelle)
aber es gibt da eben noch 
z.B.:
>>> print (string.ascii_uppercase, string.ascii_lowercase)
ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz
>>>

und damit die Verschiebung/Substitution bewerkstelligen.

noch eine Anmerkung:
in der Kryptographie scheint es ueblich zu sein, Klartext in Kleinbuchstaben und ciphertext in Grossbuchstaben zu halten!
Ich habe mich nicht daran gehalten.

Es gibt zwei Bestandteile unseres Programms:
a) Schluessel
b) Algorithmus

ad a)
der Schluessel ergibt sich aus unserer Zeichenkette, in der wir ein Zeichen des Klartextes (oder auch beim deChiffrieren des Ciphertextes) in dieser Zeichenkette aufspueren, und dann eine gewisse Stellenanzahl verschieben.
Die Zeichen in der Zeichenkette MUESSEN NICHT brav in alphabetischer Reihenfolge sein - sondern in IRGENDEINER Reihenfolge - diesen SCHLUESSEL (key) haben dann die Verschluesseler und die Entschluesseler.
ad b)
der Algorithmus ist nun, welche Zeichen ein Zeichen im Klartext ersetzen. In unserem Fall hier heisst unser Algorithmus 'Caesar-Algorithmus'

Es gibt da noch den (Grund)Satz von Kerkhoff Nieuwenhof:
"Die Sicherheit eines Kryptosystems darf nicht von der Geheimhaltung des Algorithmus abhaengen. Die Sicherheit gruendet sich nur auf die Geheimhaltung des Schluessels"

Wenn wir den Schluessel brav alphabetisch (bzw. in Reihenfolge der ASCII-Tabelle) anwenden, ergen sich len(key)-1 moegliche Schluessel
Bauen wir den Schluessel in einer beliebigen Reihenfolge, dann ergeben sich viel viel mehr Moeglichkeiten ( len(key)! -> Faktorielle von len(key) - das ist 1*2*3*...*len(key) moegliche Schluessel)
und wenn dann der Cracker jede Sekunde einen Schluessel ausprobiert, dann braucht er 'weig' und noch einen Tag! ;-)
'''

import os.path
import sys
import string
from time import sleep

__author__ = 'michael poeltl'
__copyright__ = 'GPLv2'
__version__ = '0.03'

ZEICHEN = string.punctuation[:15] \
          + string.digits \
          + string.punctuation[15:] \
          + string.ascii_uppercase \
          + string.ascii_lowercase

def caesarz(z,rueckweite=0):
    '''nimmt eine Textzeile entgegen und baut sie um nach caesar
    '''
    satz = ''
    for c in z:
        #handling fuer alle unbekannten Zeichen
        if c not in ZEICHEN:
            satz += c
        #handling fuer bekannte Zeichen
        else:
            satz += ZEICHEN[(ZEICHEN.index(c)+rueckweite)%len(ZEICHEN)]
    return satz

def caesarf(f, rueckweite=0):
    '''nimmt file entgegen und wendet caesar an
    '''
    L = []
    for zeile in open(f).readlines():
        L.append(caesarz(zeile, rueckweite))
    return L
 
def main():
    #krach wenn kein Parameter uebergeben wurde
    # muss man es krachen lassen, oder sollte man nicht lieber
    # Programm einfach so enden lassen mit 'Noting to do!
    # ?
    if len(sys.argv) == 1:
        print ("Parameter fehlt. Nothing to do!", file=sys.stderr)
        sys.exit(1)
    print ('Verschluesseln durch Substitution nach Caesar\n')
    sleep(1)
    while True:
        rueck = input('Wie weit soll nach rechts verschoben werden? ')
        if rueck.isdigit():
            rueck = int(rueck)
            break
        else:
            print ("Falsche Eingabe. Bitte noch einmal!")
    if len(sys.argv) == 2:
        if os.path.isfile(sys.argv[1]):
            print ('\n'.join(caesarf(sys.argv[1], rueck)))
        else:
            print (caesarz(sys.argv[1], rueck))
    else:
        print (caesarz(' '.join(sys.argv[1:]), rueck))

if __name__ == '__main__':
    main()
