#! /usr/bin/env python3

'''
caesar4.py
hier nun eine Weiterentwicklung des Schluessels in unserer caesar-Serie.
Bislang war es so, dass wir einen 'einfachen' Schluessel bauten - diesen Schluessel
mussten wir 'irgendwie' dem Empfaenger mitteilen, damit er die verschluesselte
Nachricht dechiffrieren konnte.

Diesmal reicht ein Geheimwort, damit der Empfaenger des Schluessel kennt, und
das funktioniert so:
Annahme:
Schluesselwort ist Michael Poeltl

dann machen wir das Leerzeichen weg, key in uppercase (damit wir dem ueblichen
Trott wenigstens ein Mal folgen, und ausserdem geht es ja nicht um korrekte Gross-/
Kleinschreibung), schmeissen moegliche Dubletten 'raus, und ergaenzen mit dem
Rest des Alphabets ab letztem Buchstaben

also
>>> from string import ascii_uppercase
>>> name = 'Michael Poeltl'
>>> key = ''
>>> for c in ''.join(name.split()).upper():
...     if c not in key:
...         key += c
...
>>> for c in ascii_uppercase[ascii_uppercase.find(key[-1])+1:] + ascii_uppercase[:ascii_uppercase.find(key[-1])]:
...     if c not in key:
...         key += c
...
>>> print (key)
MICHAELPOTUVWXYZBDFGJKNQRS
>>>

und so kann sich der Empfaenger mit dem Wissen umd das SCHLUESSELWORT den key
zum Verschluesseln und Entschluesseln selber zusammenstellen

und das ist das, was wir hier durchexerzieren.
Die Schrittweite ist wiederum 3 Zeichen --> Caesar-Algorithmus

Beispiel eines Programmaufrufs:
?> ~/work/uniweb/py/bsp/caesar4.py mIchaeL POelTl
Chiffrieren oder Dechiffrieren? [c/d]a
Gib das Schluesselwort ein:
Chiffrieren nach Caesar

HAELPOT UVOTWT

?> ~/work/uniweb/py/bsp/caesar4.py HAELPOT UVOTWT
Chiffrieren oder Dechiffrieren? [c/d]d
Gib das Schluesselwort ein:
Dechiffrieren nach Caesar

michael poeltl

?>
'''
__author__ = 'michael poeltl'
__copyright__ = 'GPLv2'
__version__ = '0.04'

import sys
import os.path
from time import sleep
from string import ascii_uppercase
from getpass import getpass

SCHRITTWEITE = SW = 3

def schluessel(swort):
    '''baut den key abhaengig von einem Schluesselwort (swort)
    '''
    key = ''
#ich setze absichtlich noch keine list-comprehension ein
    for c in ''.join(swort.split()).upper():
        if c not in key:
            key += c
    for c in ascii_uppercase[ascii_uppercase.find(key[-1])+1:]+ascii_uppercase[:ascii_uppercase.find(key[-1])]:
        if c not in key:
            key += c
    return key        

def caesarz(z, sch, art='c'):
    '''nimmt eine Textzeile entgegen und baut sie um nach caesar
    '''
    satz = ''
    for c in z.upper():
        #handling fuer alle unbekannten Zeichen
        if c not in sch:
            satz += c
        #handling fuer bekannte Zeichen
        else:
            satz += sch[(sch.find(c)+SW)%len(sch)]
    if art == 'c':
        return satz
    else:
        return satz.lower()

def caesarf(f, sch, art='c'):
    '''nimmt file entgegen und wendet caesar an
    '''
    L = []
    for zeile in open(f).readlines():
        L.append(caesarz(zeile, sch,art))
    return L

def main():
    if len(sys.argv) == 1:
        #nothing to do
        sys.exit(0)
    c_oder_d = input("Chiffrieren oder Dechiffrieren? [c/d]").lower()[0]
    if c_oder_d not in 'cd':
        c_oder_d = 'c'
    thekey = schluessel(getpass("Gib das Schluesselwort ein: "))
    if c_oder_d == 'c':
#nach sys.stderr falls Ausgabe in ein file umgelenkt werden soll
        print ('Chiffrieren nach Caesar', file=sys.stderr)
    else:
        print ('Dechiffrieren nach Caesar', file=sys.stderr)
        thekey = thekey[::-1]
    sleep(1)
    if os.path.isfile(sys.argv[1]):
        print ('\n'.join(caesarf(sys.argv[1], thekey, c_oder_d)))
    else:
        print (caesarz(' '.join(sys.argv[1:]), thekey, c_oder_d))

if __name__ == '__main__':
    main()
