'''
everything started with pyame (vectors in 2d and 3d)
names for stuff in 2d will have the ending 2d
names for stuff in 3d will have the ending 3d

any explanations/reminders are documented in mygeometry.txt
'''

import math

__state__ = 'development'
__started__ = '26th of june 2014'
__version__ = '0.01'

class Vector2d:
    def __init__(self, x=0, y=0, name=''):
        self.x = x
        self.y = y
        self.name = name
    def __repr__(self):
        return '{}({},{})'.format(self.name, self.x, self.y)
    @classmethod
    def twopoints(cls, p1, p2, name=''):
        '''
        point1 to point2 returns a vector2d instance
        '''
        return cls(p2[0]-p1[0], p2[1]-p1[1],name)

    def __add__(self, second):
        return Vector2d(self.x+second.x, self.y+second.y)

    def __sub__(self, second):
        return Vector2d( self.x-second.x, self.y-second.y )

    def __neg__(self):
        return Vector2d( -self.x, -self.y )

    def __mul__(self, skalar):
        assert isinstance( skalar, (int,float) )
        self.x *= skalar
        self.y *= skalar
        return self

    def __truediv__(self, skalar):
        assert isinstance( skalar, (int, float) )
        self.x /= skalar
        self.y /= skalar
        return self

    def __bool__(self):
        return self.x != 0 or self.y != 0#returns True

    def __eq__(self, second):
        assert isinstance(second, Vector2d)
        one = self.normalized()
        two = second.normalized()
        if round(one.x,14) == round(two.x,14) and round(one.y,14) == round(two.y,14):
            return True
        else:
            return False

    def __ne__(self, second):
        assert isinstance(second, Vector2d)
        return not self.__eq__(second)

    def get_magnitude(self):
        return math.sqrt( math.pow(self.x,2) + math.pow(self.y,2))

    def normalize(self):
        '''
        vector becomes unit vector
        '''
        if self:
            distance = d = self.get_magnitude()
            self.x /= d
            self.y /= d
            return self
        else:
            return self

    def normalized(self):
        '''
        unit vector of vector will be returned
        '''
        if self:
            distance = d = self.get_magnitude()
            return Vector2d( self.x/d, self.y/d)
        else:
            return self

    def dot(self, second):
        assert isinstance(second, Vector2d)
        return self.x*second.x + self.y*second.y

    def is_orthogonal(self, second):
        assert isinstance(second, Vector2d)
        if self.dot(second):
            return False
        else:
            return True #result was 0

    def angle(self, second):
        assert isinstance(second, Vector2d)
        return math.acos(self.dot(second) / (self.get_magnitude()*second.get_magnitude()))
        
    def angle_grad(self, second):
        assert isinstance(second, Vector2d)
        return (self.angle(second) / (2*math.pi))*360

    def get_pos(self):
        return (self.x, self.y)


class Point2d:
    pass
